/**
 * The main pupose of this class is to provide a better control of an animation.
 * For example, when an animation is played, if some frames are lost the class compensates automatically
 * @author SamsamTS
 * @see Animation
 */
 
import com.samsamts.Events.EventWrapper;
import com.samsamts.Events.EventHandler;
import com.samsamts.AnimationCtrl.Animation;
import com.samsamts.AnimationCtrl.AnimationEventArgs;

class com.samsamts.AnimationCtrl.AnimationClip extends MovieClip
{
	//{ Author properties
	public static var className:String = "AnimationClip";
	public static var classPackage:String = "com.samsamts";
	public static var version:String = "1.0.0";
	public static var author:String = "SamsamTS";
	public static var link:String = "http://www.samsamts.com/blog/";
	//}
	
	//{ Events
	public var OnData:EventWrapper;
	public var OnDragOut:EventWrapper;
	public var OnDragOver:EventWrapper;
	public var OnKillFocus:EventWrapper;
	public var OnPress:EventWrapper;
	public var OnRelease:EventWrapper;
	public var OnReleaseOutside:EventWrapper;
	public var OnRollOut:EventWrapper;
	public var OnRollOver:EventWrapper;
	public var OnSetFocus:EventWrapper;
	public var OnUnload:EventWrapper;
	//}
	
	//{ Static
	/**
	* Convert a MovieClip to an AnimationClip
	* @param	mc The MovieClip to convert
	* @param	deep Optional. If set to true then all MovieClip in mc will be converted too.
	* @return	Reference to rhe converted MovieClip
	*/
	public static function ConvertMovieClip(mc:MovieClip, deep:Boolean):AnimationClip
	{
		mc.__proto__ = AnimationClip.prototype;
		Function(AnimationClip).apply(mc);
		if(deep)
		{
			for(var submc:String in mc)
			{
				if(mc[submc] instanceof MovieClip)
					ConvertMovieClip(mc[submc]);
			}
		}
		return AnimationClip(mc);
	}
	//}

	//{ Public
	/**
	* Tell if the animation plays in loop, true by default.
	*/
	public var Loop:Boolean;
	/**
	* Create an empty AnimationClip
	* @param	name The name of the clip
	* @param	depth The depth of the clip
	* @return	The created AnimationClip
	*/
	public function createEmptyAnimationClip(name:String, depth:Number):AnimationClip
	{
		var mc:MovieClip = super.createEmptyMovieClip.apply(this, arguments);
		return ConvertMovieClip(mc);
	}
	/**
	* Create an AnimationClip from a MovieClip linked in the library
	* @param	id	The link identifier
	* @param	name	The name of the clip
	* @param	depth	The depth of the clip
	* @param	initClass	Optional. The class of the clip if other than AnimationClip
	* @return	The created AnimationClip
	*/
	public function attachAnimationClip(id:String, name:String, depth:Number, initClass:Function):AnimationClip
	{
		arguments.splice(3,1);
		var mc:MovieClip = super.attachMovie.apply(this, arguments);
		if(!initClass)
			initClass = AnimationClip;
		mc.__proto__ = initClass.prototype;
		initClass.apply(mc);
		for(var submc:String in mc)
		{
			if(mc[submc] instanceof MovieClip)
				ConvertMovieClip(mc[submc]);
		}
		return AnimationClip(mc);
	}
	/**
	* The animation plays reverse
	*/
	public function playReverse():Void
	{
		m_reverse = true;
		if(!m_playing)
		{
			m_playing = true;
			Animation.OnEnterFrame.Add(new EventHandler(this, OnEnterFrameAnimation));
		}
	}
	/**
	* Set an action that must be done when enter a specific frame
	* @param	index	Frame on wich the action is done
	* @param	f	Function to call
	* @return	Return false if index is invalid
	*/
	public function SetActionAtFrame(index:Number, f:Function):Boolean
	{
		index--;
		if(index < 0 || index >= _totalframes)
			return false;
		m_frameFunction[index] = f;
		return true;
	}
	public function GetActionAtFrame(index:Number):Function
	{
		return m_frameFunction[index-1];
	}
	//}
	
	//{ Overrides
	public function createTextField(instanceName:String, depth:Number, x:Number, y:Number, width:Number, height:Number):TextField
	{
		super.createTextField.apply(this, arguments);
		return this[instanceName];
	}

	public function stop():Void
	{
		if(m_playing)
		{
			m_playing = false;
			Animation.OnEnterFrame.Remove(new EventHandler(this, OnEnterFrameAnimation));
		}
	}

	public function play():Void
	{
		m_reverse = false;
		if(!m_playing)
		{
			m_playing = true;
			Animation.OnEnterFrame.Add(new EventHandler(this, OnEnterFrameAnimation));
		}
	}

	public function gotoAndStop(frame:Object):Void
	{
		stop();
		super.gotoAndStop.apply(this, arguments);
	}

	public function gotoAndPlay(frame:Object):Void
	{
		super.gotoAndStop.apply(this, arguments);
		play();
	}

	public function nextFrame():Void
	{
		stop();
		super.nextFrame.apply(this);
	}
	
	public function toString():String
	{
		return "[AnimationClip : "+_name+"]";
	}
	//}
	
	//{ Private
	private var m_playing:Boolean;
	private var m_reverse:Boolean;
	private var m_idInterval:Number;
	private var m_frameFunction:Array;
	
	private var onEnterFrame:Function;
	private var onKeyDown:Function;
	private var onKeyUp:Function;
	private var onMouseDown:Function;
	private var onMouseMove:Function;
	private var onMouseUp:Function;

	private function AnimationClip()
	{
		OnData				= new EventWrapper(this, "onData");
		OnDragOut			= new EventWrapper(this, "onDragOut");
		OnDragOver			= new EventWrapper(this, "onDragOver");
		OnKillFocus			= new EventWrapper(this, "onKillFocus");
		OnPress				= new EventWrapper(this, "onPress");
		OnRelease			= new EventWrapper(this, "onRelease");
		OnReleaseOutside	= new EventWrapper(this, "onReleaseOutside");
		OnRollOut			= new EventWrapper(this, "onRollOut");
		OnRollOver			= new EventWrapper(this, "onRollOver");
		OnSetFocus			= new EventWrapper(this, "onSetFocus");
		OnUnload			= new EventWrapper(this, "onUnload");

		Loop = true;
		m_frameFunction = new Array();
		
		if(_totalframes > 1)
		{
			super.stop.apply(this);
			Animation.OnEnterFrame.Add(new EventHandler(this, OnEnterFrameAnimation));
			Animation.OnPause.Add(new EventHandler(this, OnPause));
			Animation.OnResume.Add(new EventHandler(this, OnResume));
			m_playing = true;
			m_reverse = false;
		}
	}

	private function OnEnterFrameAnimation(sender:Object, arg:AnimationEventArgs):Void
	{
		var nb:Number = arg.FramesLost+1;
		while(nb-- > 0)
		{
			if(!m_playing)
				return;
			if(m_reverse)
			{
				if(_currentframe == 1 && Loop)
					super.gotoAndStop.apply(this, [_totalframes]);
				else
					super.prevFrame.apply(this);
			}
			else
			{
				if(_currentframe == _totalframes && Loop)
					super.gotoAndStop.apply(this, [1]);
				else
					super.nextFrame.apply(this);
			}
			
			if(m_frameFunction[_currentframe-1] != undefined)
			{
				Function(m_frameFunction[_currentframe-1]).apply(this);
			}
		}
	}
	
	private function OnPause():Void
	{
		this.enabled = false;
	}
	private function OnResume():Void
	{
		this.enabled = true;
	}
	//}
}